require 'spec_helper'

describe "Inkan" do
  describe '.legitimate?' do
    context "without a hashbang" do
      let(:content) { 'foo bar baz' }

      it "returns true if the file contents match the SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.puts "# Generated by Inkan: #{Digest::SHA1.hexdigest(content)}"
          file.print content
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_true
      end

      it "returns false if the file contents don't match the SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.puts "# Generated by Inkan: #{Digest::SHA1.hexdigest(content)}"
          file.puts content
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_false
      end

      it "returns false if there is no SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.print content
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_false
      end
    end
    context "with a hashbang" do
      let(:content) { "#!/bin/sh\nfoo bar baz" }

      it "returns true if the file contents match the SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.puts '#!/bin/sh'
          file.puts "# Generated by Inkan: #{Digest::SHA1.hexdigest(content)}"
          file.print 'foo bar baz'
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_true
      end

      it "returns false if the file contents don't match the SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.puts '#!/bin/sh'
          file.puts "# Generated by Inkan: #{Digest::SHA1.hexdigest(content)}"
          file.puts 'foo bar baz'
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_false
      end

      it "returns false if there is no SHA" do
        open('/tmp/spec.txt', 'w') do |file|
          file.print content
        end

        Inkan.legitimate?('/tmp/spec.txt').should be_false
      end
    end
  end

  describe '.seal' do
    it "should write the file after closing the block" do
      Inkan.seal('/tmp/spec.txt') do |inkan|
        inkan.print 'foo bar baz'
      end

      contents = open('/tmp/spec.txt').read
      contents.should match(/foo bar baz$/)
      contents.should match(/^# Generated by Inkan/)
    end
  end

  describe '.render' do
    it "should return the result after closing the block" do
      result = Inkan.render do |inkan|
        inkan.print 'foo bar baz'
      end

      result.should match(/foo bar baz$/)
      result.should match(/^# Generated by Inkan/)
    end
  end

  describe '#seal' do
    let(:inkan) { Inkan.new('/tmp/spec.txt') }

    it "writes out the contents of the buffer" do
      inkan.print "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/foo bar baz$/)
    end

    it "adds the credit line to the top of the file" do
      inkan.print "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/^# Generated by Inkan/)
    end

    it "adds the SHA to the top of the file" do
      inkan.print "foo bar baz"
      inkan.seal

      sha = Digest::SHA1.hexdigest("foo bar baz")

      open('/tmp/spec.txt').read.should match(/#{sha}/)
    end
  end

  describe '#render' do
    let(:inkan) { Inkan.new(nil) }

    before { shell "rm -f /tmp/spec.txt" }

    it "writes out the contents of the buffer" do
      inkan.print "foo bar baz"
      inkan.render.should match(/foo bar baz$/)
    end

    context "when there is no hashbang" do
      before { inkan.print "foo bar baz" }
      it "adds the credit line to the top of the output" do
        inkan.render.split("\n").first.should match(/^# Generated by Inkan/)
      end
      it "adds the SHA to the top of the output" do
        sha = Digest::SHA1.hexdigest("foo bar baz")
        inkan.render.split("\n").first.should match(/#{sha}/)
      end
    end

    context "when there is a hashbang" do
      before { inkan.print "#!/bin/sh\nfoo bar baz" }
      it "leaves the hashbang at the top of the file" do
        inkan.render.split("\n")[0].should == '#!/bin/sh'
      end
      it "adds the credit line to the top of the output below the hashbang" do
        inkan.render.split("\n")[1].should match(/^# Generated by Inkan/)
      end
      it "adds the SHA to the top of the output below the hashbang" do
        sha = Digest::SHA1.hexdigest("#!/bin/sh\nfoo bar baz")
        inkan.render.split("\n")[1].should match(/#{sha}/)
      end
    end

    it "doesn't write the file" do
      inkan.print "foo bar baz"
      inkan.render
      File.exists?('/tmp/spec.txt').should be_false
    end
  end

  describe '#puts' do
    let(:inkan) { Inkan.new('/tmp/spec.txt') }

    it "adds an extra new line to the output" do
      inkan.puts "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/foo bar baz\n$/)
    end
  end

  describe '#credit' do
    let(:inkan) { Inkan.new('/tmp/spec.txt') }

    it "defaults to 'Generated by Inkan'" do
      inkan.credit.should == 'Generated by Inkan'
    end

    it "should pass through changes to the file comment" do
      inkan.credit = "Pat's Magic Code"
      inkan.print "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/^# Pat's Magic Code/)
    end
  end

  describe '#comment' do
    let(:inkan) { Inkan.new('/tmp/spec.txt') }

    it "defaults to a hash symbol" do
      inkan.comment.should == '#'
    end

    it "should pass through changes to the file credit" do
      inkan.comment = "//"
      inkan.print "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/^\/\/ Generated by Inkan/)
    end
  end

  describe '#comment_suffix' do
    let(:inkan) { Inkan.new('/tmp/spec.txt') }

    it "defaults to a blank string" do
      inkan.comment_suffix.should == ''
    end

    it "should pass through changes to the file comment" do
      inkan.comment_suffix = "*/"
      inkan.print "foo bar baz"
      inkan.seal

      open('/tmp/spec.txt').read.should match(/\*\/\n/)
    end
  end
end
